#include <stdio.h>
#include <stdlib.h>
#include <pwd.h>
#include <sys/types.h>
#include <rsbac/types.h>
#include <rsbac/aci_data_structures.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>

int main(int argc, char ** argv)
{
  int attr_list[RSBAC_USER_NR_ATTRIBUTES] = RSBAC_USER_ATTR_LIST;
  int res = 0;
  char tmp1[80],tmp2[120];
  int j;
  union rsbac_attribute_value_t value;
  union rsbac_target_id_t tid;
  enum rsbac_attribute_t attr;
  struct passwd * user_info_p;
  int inherit = 0;
  char * progname = argv[0];
  
  if ((argc > 1) && (!strcmp("-e",argv[1])) )
    {
      inherit=1;
      argv++;
      argc--;
    }  
  
  if (argc == 3)
    {
      value.dummy = -1;
      if(   (!(user_info_p = getpwnam(argv[1])))
         && (   (!(user_info_p = getpwuid(strtol(argv[1],0,0))))
             || (!(user_info_p->pw_uid) && strcmp("0",argv[1])) )
        )
          {
            fprintf(stderr, "Invalid user %s!\n", argv[1]);
            exit(1);
          }
      tid.user = user_info_p->pw_uid;
      if(!strcmp("user_nr",argv[2]))
        {
          printf("%i\n", user_info_p->pw_uid);
          exit(0);
        }
      if(!strcmp("user_name",argv[2]))
        {
          printf("%s\n", user_info_p->pw_name);
          exit(0);
        }
      if(!strcmp("full_name",argv[2]))
        {
          printf("%s\n", user_info_p->pw_gecos);
          exit(0);
        }

      attr = get_attribute_nr(argv[2]);
      if(attr == A_none)
        {
          fprintf(stderr, "%s: Invalid Attribute %s!\n", progname, argv[2]);
          exit(1);
        }

      res = rsbac_get_attr(T_USER, &tid, attr, &value, inherit);
      if(res)
        {
          if((res > -RSBAC_EPERM) || (res < -RSBAC_EDECISIONMISMATCH))
            {
              fprintf(stderr, "Error: %s\n", strerror(-res));
            }
          else
            {
              get_error_name(tmp1,res);
              fprintf(stderr, "Error: %s\n", tmp1);
            }
          exit(1);
        }
      else
        {
          printf("%i\n",value.dummy);
          exit(0);
        }
    }
  else
    {
      printf("%s (RSBAC %s)\n***\n", progname, VERSION);
      printf("Use: %s [-e] user attribute\n\n", progname);  
      printf("- -e = show effective (maybe inherited) value, not real\n");
      printf("- attribute (string) and returned value = see following list:\n");
      for (j=0;j<RSBAC_USER_NR_ATTRIBUTES;j++)
        {
          get_attribute_name(tmp1,attr_list[j]);
          get_attribute_param(tmp2,attr_list[j]);
          printf("%s \t%s\n",tmp1,tmp2);
        }
    }
  exit(1);
}
